require_dependency 'application'
require_dependency 'my_controller'

module MailConfiguratorMyControllerPatch
  def self.included(base) # :nodoc:
    base.extend(ClassMethods)

    base.send(:include, InstanceMethods)

    # Same as typing in the class
    base.class_eval do
      unloadable # Send unloadable so it will not be unloaded in development
      alias_method_chain :account, :new_user_preferences_hook
    end
  end

  module ClassMethods
  end

  module InstanceMethods
    # Edit user's account
    def account_with_new_user_preferences_hook
      @user = User.current
      @pref = @user.pref
      if request.post?
        @user.attributes = params[:user]
        @user.mail_notification = (params[:notification_option] == 'all')
        @user.pref.attributes = params[:pref]
        @user.pref[:no_self_notified] = (params[:no_self_notified] == '1')
        call_hook(:my_controller_new_user_preferences, { :params => params, :user_pref => @user.pref })
        if @user.save
          @user.pref.save
          @user.notified_project_ids = (params[:notification_option] == 'selected' ? params[:notified_project_ids] : [])
          set_language_if_valid @user.language
          flash[:notice] = l(:notice_account_updated)
          redirect_to :action => 'account'
          return
        end
      end
      @notification_options = [[l(:label_user_mail_option_all), 'all'],
                               [l(:label_user_mail_option_none), 'none']]
      # Only users that belong to more than 1 project can select projects for which they are notified
      # Note that @user.membership.size would fail since AR ignores :include association option when doing a count
      @notification_options.insert 1, [l(:label_user_mail_option_selected), 'selected'] if @user.memberships.length > 1
      @notification_option = @user.mail_notification? ? 'all' : (@user.notified_projects_ids.empty? ? 'none' : 'selected')
    end
  end
end

# Add module to My
MyController.send(:include, MailConfiguratorMyControllerPatch)